package com.insomnia.parser;

import java.util.ArrayList;
import java.util.HashMap;

public class Parser {

   public static ArrayList<String> fixArgs(String[] args){
        ArrayList<String> args2 = new ArrayList<>();
        String f = "";
        for (String arg : args){
            if (arg.trim().startsWith("-")){
                if (!f.isEmpty()) args2.add(f.trim());
                f = arg.trim()+" ";
            }else{
                f = f + arg.trim()+" ";
            }
        }
        if (!f.isEmpty()) args2.add(f.trim());
        return args2;
    }

    public static RequestData parse(String[] args){
        RequestData requestData = new RequestData();

        for (String arg: fixArgs(args)) {
            String a = findArg(arg).trim();

            String paramName,paramValue="";
            if (a.contains(" ")){
                paramName = a.substring(0,a.indexOf(" ")).trim();
                paramValue = findParamValue(a.substring(a.indexOf(" ")+1).trim());
            }else{
                paramName = a;
            }

            switch (paramName.toLowerCase()) {
                case "url":
                    requestData.setUrl(paramValue);
                    break;

                case "method":
                case "m":
                    requestData.setMethod(findMethod(paramValue));
                    break;

                case "headers":
                case "h":
                    if (paramValue.length()>2){
                        HashMap<String,String> headers = new HashMap<>();
                        if (!paramValue.contains(";")){
                            String[] h1 = paramValue.split(":");
                            headers.put(h1[0],h1[1]);
                        }else{
                            String[] mHeaders = paramValue.split(";");
                            for (String h : mHeaders){
                                String[] h0 = h.split(":");
                                headers.put(h0[0],h0[1]);
                            }
                        }
                        requestData.setHeaders(headers);
                    }
                    break;

                case "i":
                    requestData.setShowResponseHeaders(true);
                    break;

                case "f":
                    requestData.setFollowRedirects(true);
                    break;

                case "output":
                case "o":
                    requestData.setOutput(  paramValue.isEmpty() ? "-t" : paramValue);
                    break;

                case "save":
                case "s":
                    requestData.setSaveRequest(true);
                    break;

                case "json":
                case "j":
                    requestData.setJson(true);
                    break;

                case "data":
                case "d":
                    requestData.setBody(paramValue);
                    break;

                case "upload":
                    requestData.setUploadPath(paramValue);
            }
        }

        return requestData;
    }


    static String findMethod(String v){
        switch (v.toUpperCase().trim()){
            case "GET":
            case "POST":
            case "DELETE":
            case "PUT":
                return v.toUpperCase().trim();

            default:
                return RequestData.GET;
        }
    }

    static String findParamValue(String v){
        if (v.startsWith("\"") && v.endsWith("\"")){
            return v.substring(1,v.length()-1);
        }
        return v;
    }

    static String findArg (String v){
        if (v.startsWith("--")){
            return v.substring(2);
        }else if (v.startsWith("-")){
            return v.substring(1);
        }
        return v;
    }
}
